Package: mpath
Title: Regularized Linear Models
Version: 0.4-2.27
Date: 2026-07-12
Authors@R: c(
      person(given = "Zhu", family = "Wang", role = c("aut", "cre"), email = "zwang145@uthsc.edu"),
      person(given = "Achim", family = "Zeileis", role = "aut"),
      person(given = "Simon", family = "Jackman", role = "aut"),
      person(given = "Brian", family = "Ripley", role = "aut"),
      person(given = "Patrick", family = "Breheny", role = "aut")
      )
Maintainer: Zhu Wang <zwang145@uthsc.edu>
Description: Algorithms compute robust estimators for loss functions in the concave convex (CC) family by the iteratively reweighted convex optimization (IRCO), an extension of the iteratively reweighted least squares (IRLS). The IRCO reduces the weight of the observation that leads to a large loss; it also provides weights to help identify outliers. Applications include robust (penalized) generalized linear models and robust support vector machines. The package also contains penalized Poisson, negative binomial, zero-inflated Poisson, zero-inflated negative binomial regression models and robust models with non-convex loss functions. Wang et al. (2014) <doi:10.1002/sim.6314>,
      Wang et al. (2015) <doi:10.1002/bimj.201400143>,
      Wang et al. (2016) <doi:10.1177/0962280214530608>,
      Wang (2021) <doi:10.1007/s11749-021-00770-2>,
      Wang (2024) <doi:10.1111/anzs.12409>.
Depends: R (>= 3.5.0), methods, glmnet
Imports: MASS, pscl, numDeriv, foreach, doParallel, bst, WeightSVM
Suggests: R.rsp
VignetteBuilder: R.rsp
License: GPL-2
URL: https://github.com/zhuwang46/mpath
BugReports: https://github.com/zhuwang46/mpath/issues
Packaged: 2026-07-12 20:40:57 UTC; zwang145
NeedsCompilation: yes
RoxygenNote: 7.1.1
Author: Zhu Wang [aut, cre],
  Achim Zeileis [aut],
  Simon Jackman [aut],
  Brian Ripley [aut],
  Patrick Breheny [aut]
Repository: CRAN
Date/Publication: 2026-07-12 21:50:02 UTC
