useDynLib(mpath)

export(glmreg, glmreg_fit, glmregNB, zipath, llfun, predictzeroinfl1, cv.folds, cv.glmreg, cv.glmreg_fit, cv.glmregNB, cv.zipath, tuning.zipath, pval.zipath, conv2glmreg, conv2zipath, estfunReg, sandwichReg, breadReg, meatReg, hessianReg, se, be.zeroinfl, rzi, ncl, nclreg, ncl_fit, nclreg_fit, cv.nclreg, cv.nclreg_fit, stan, zipath_fit, irglm, irglmreg, irglmreg_fit, irsvm, irsvm_fit, cv.irsvm, cv.irsvm_fit, loss2, loss2_irsvm, loss3, cv.irglmreg, cv.irglmreg_fit, compute_wt, update_wt, compute_g,
gfunc, cfun2num, y2num, y2num4glm, check_s)

import("methods")
import("stats")
import("glmnet")
importFrom("MASS", "theta.ml")
importFrom("MASS", "glm.nb")
importFrom("MASS", "rnegbin")
importFrom("MASS", "negative.binomial")
importFrom("pscl", "zeroinfl")
importFrom("pscl", "predprob")
importFrom("numDeriv", "hessian")
importFrom("graphics", "plot", "segments")
importFrom("graphics", "axis", "matplot", "text")
importFrom("utils", "capture.output")
importFrom("foreach", "foreach")
importFrom("foreach", "%dopar%")
importFrom("foreach", "%:%")
importFrom("doParallel", "registerDoParallel")
importFrom("doParallel", "stopImplicitCluster")
importFrom("bst", "loss")
importFrom("bst", "bfunc")
importFrom("bst", "gradient")
importFrom("bst", "bst_control")
importFrom("bst", "bst")
importFrom("bst", "evalerr")
importFrom("WeightSVM", "wsvm")

S3method(glmreg, formula)
S3method(glmreg, matrix)
S3method(glmreg, default)
S3method(coef, glmreg)
S3method(deviance, glmreg)
S3method(plot, glmreg)
S3method(predict, glmreg)
S3method(logLik, glmreg)
S3method(AIC, glmreg)
S3method(BIC, glmreg)
S3method(predprob, glmreg)
S3method(cv.glmreg, formula)
S3method(cv.glmreg, matrix)
S3method(cv.glmreg, default)
S3method(plot, cv.glmreg)
S3method(predict, cv.glmreg)
S3method(coef, cv.glmreg)
S3method(summary, glmregNB)
S3method(print, summary.glmregNB)

S3method(zipath, formula)
S3method(zipath, matrix)
S3method(zipath, default)
S3method(coef, zipath)
S3method(plot, zipath)
S3method(predict, zipath)
S3method(summary, zipath)
S3method("residuals", "zipath")
S3method("fitted", "zipath")
S3method(logLik, zipath)
S3method(AIC, zipath)
S3method(BIC, zipath)
S3method(estfunReg, zipath)
S3method(hessianReg, zipath)
S3method(breadReg, zipath)
S3method(meatReg, zipath)
S3method(se, zipath)
S3method(predprob, zipath)
S3method(cv.zipath, formula)
S3method(cv.zipath, matrix)
S3method(cv.zipath, default)
S3method(predict, cv.zipath)
S3method(coef, cv.zipath)
S3method(plot, cv.zipath)
S3method(print, summary.zipath)

S3method(ncl, formula)
S3method(ncl, matrix)
S3method(ncl, default)
S3method(predict, ncl)
S3method(nclreg, formula)
S3method(nclreg, matrix)
S3method(nclreg, default)
S3method(predict, nclreg)
S3method(coef, nclreg)
S3method(cv.nclreg, formula)
S3method(cv.nclreg, matrix)
S3method(cv.nclreg, default)
S3method(plot, cv.nclreg)
S3method(coef, cv.nclreg)

S3method(irglm, formula)
#S3method(irglm, matrix)
#S3method(irglm, default)
S3method(predict, irglm)
S3method(irglmreg, formula)
S3method(irglmreg, matrix)
S3method(irglmreg, default)
S3method(predict, irglmreg)
S3method(coef, irglmreg)
S3method(cv.irglmreg, formula)
S3method(cv.irglmreg, matrix)
S3method(cv.irglmreg, default)
S3method(plot, cv.irglmreg)
S3method(coef, cv.irglmreg)
S3method(irsvm, formula)
S3method(irsvm, matrix)
S3method(irsvm, default)
S3method(cv.irsvm, formula)
S3method(cv.irsvm, matrix)
S3method(cv.irsvm, default)
